new;
library tsm,optmum,spt,pgraph;
TSmset;
SPTset;

cls;

rndseed 123;

a = 0.95; b = 0.10; sigma = 0.2; lambda = 0.05;
theta = 0.5;

bprime = b - lambda*sigma/a;


proc (3) = Theta_Euler(r0,t0,T,N,M,theta);
  local k,tau,u,S1,Si1,S2,Si2,w1,w2,w3,i;

  k = (T - t0)/(N-1);
  tau = seqa(0,k,N);
  u = rndn(N,M)*sqrt(k);
  S1 = zeros(N,M);
  S2 = zeros(N,M);

  Si1 = r0.*ones(1,M);
  S1[1,.] = Si1;

  Si2 = r0.*ones(1,M);
  S2[1,.] = Si2;

  w1 = 1 + theta*a*k;
  w2 = 1 - (1-theta)*a*k;
  w3 = a*k*bprime;

  i = 2;
  do until i > N;
    Si1 = ( w2*Si1 + w3 + sigma*u[i,.] )/w1;
    S1[i,.] = Si1;
    Si2 = ( w2*Si2 + w3 - sigma*u[i,.] )/w1;
    S2[i,.] = Si2;

    i = i + 1;
  endo;

  retp(tau,S1,S2);
endp;


proc intTrapeze(S,k);
  local N,I;

  N = rows(S);
  I = k*(sumc(S[2:N-1,.]) + 0.5*(S[1,.]' + S[N,.]') );

  retp(I);
endp;


t0 = 0;
T = 1;
r0 = 0.10;

M = 500; 
Nr = 250;

N = 101;
k = (T-t0)/(N-1);

P1 = zeros(Nr,1);
P2 = zeros(Nr,1);
P3 = zeros(Nr,1);
P4 = zeros(Nr,1);

j = 1;
do until j > Nr;

  {tau,S,Sstar} = Theta_Euler(r0,t0,T,N,M,theta);
  S1 = S[.,1:25];
  Sstar1 = Sstar[.,1:25];
  I = 0.5*( exp(-intTrapeze(S1,k)) + exp(-intTrapeze(Sstar1,k)) );
  P1[j] = meanc(I);
  S1 = S[.,1:100];
  Sstar1 = Sstar[.,1:100];
  I = 0.5*( exp(-intTrapeze(S1,k)) + exp(-intTrapeze(Sstar1,k)) );
  P2[j] = meanc(I);
  S1 = S[.,1:250];
  Sstar1 = Sstar[.,1:250];
  I = 0.5*( exp(-intTrapeze(S1,k)) + exp(-intTrapeze(Sstar1,k)) );
  P3[j] = meanc(I);
  S1 = S[.,1:500];
  Sstar1 = Sstar[.,1:500];
  I = 0.5*( exp(-intTrapeze(S1,k)) + exp(-intTrapeze(Sstar1,k)) );
  P4[j] = meanc(I);

  locate 5,5;
  print /flush ftos(j,"Iteration no. %lf",2,0);

  j = j + 1;
endo;

{x1,d1,F1,retcode} = Kernel(P1);
{x2,d2,F2,retcode} = Kernel(P2);
{x3,d3,F3,retcode} = Kernel(P3);
{x4,d4,F4,retcode} = Kernel(P4);

{CouponZero,TauxZero} = Vasicek(r0,a,b,sigma,lambda,T-t0);

graphset;
  _pdate = ""; _pnum = 2;
  title("Acceleration de Monte Carlo");
  _plegstr = "M = 25\000M = 100\000M = 250\000M = 500";
  _plegctl = {2 5 1 3};
  _pltype = 6|1|3|4;
  xtics(0.910,0.913,0.001,10); 
  _pline = 1~5~CouponZero~0~CouponZero~5000~1~15~0;
  graphprt("-c=1 -cf=mc4.eps");
  xy(x1~x2~x3~x4,d1~d2~d3~d4);
