new;
library tsm,optmum,spt,pgraph;
TSMset;

cls;

rndseed 123456;

proc Solution(alpha,X0,MU,SIGMA,Rho,Lambda,tau);
  local r0,w1,w2,CouponZero;

  r0 = alpha'X0;
  w1 = alpha'(MU-SIGMA*LAMBDA);
  w2 = alpha'*SIGMA*Rho*SIGMA'alpha;
  
  CouponZero = exp( -R0.*tau - 0.5*w1.*(tau^2) + w2.*(tau^3)/6 );

  retp(CouponZero);
endp;


Proc (3) = SimulationTaylor(X0,Muproc,SIGMAproc,Rho,t0,T,NBpoints);
  local Muproc:proc,SIGMAproc:proc;
  local M,N,X,k,tt,Pchol,Xi,ti,i,e;

  M = rows(X0);  /* Nombre de variables d'etat */
  N = rows(Rho); /* Nombre de risques          */

  X = zeros(M,NBpoints);
  X[.,1] = X0;
  
  k = (T-t0)/(NBpoints-1);
  tt = seqa(t0,k,NBpoints);

  Pchol = chol(Rho)';
  Pchol = sqrt(k)*Pchol;

  Xi = X0;
  ti = t0;

  i = 2;
  do until i > NBpoints;
    e = rndn(N,1);
    Xi = Xi + MUproc(ti,Xi)*k + SIGMAproc(ti,Xi)*Pchol*e;
    X[.,i] = Xi;
    ti = tt[i];
    i = i + 1;
  endo;

  retp(k,tt,X');
endp;

proc intTrapeze2(S,k);
  local N,M,I,j;

  N = rows(S); M = cols(S);
  I = zeros(N,M);

  j = 2;
  do until j > N;
    I[j,.] = I[j-1,.] + 0.5*(S[j-1,.] + S[j,.])*k;
    j = j + 1;
  endo;

  retp(I);
endp;


let X0 = 0.10 0.15;
let MU = 0.3 0.5;
let SIGMA[2,2] = 0.25 0 0 0.15;
let Rho[2,2] = 1 0.5 0.5 1;
let Lambda = 0.1 0.2;
let alpha = 0.3 -0.1;

proc pMU(t,X);
  retp(MU-SIGMA*Lambda);
endp;

proc pSIGMA(t,X);
  retp(SIGMA);
endp;

proc pTaux(t,X);
  retp(alpha*X);
endp;


Ns = 100;
Nr = 1;

CZsimul = zeros(Ns,Nr);

s = 1;
do until s > Nr;
  {k,ts,Xs} = SimulationTaylor(X0,&pMu,&pSIGMA,Rho,0,5,Ns);
  Rs = Xs*alpha;
  CZsimul[.,s] = exp(-intTrapeze2(Rs,k));
  s = s + 1;
endo;
  
CZsimul1 = meanc(CZsimul');

Ns = 100;
Nr = 5000;

CZsimul = zeros(Ns,Nr);

s = 1;
do until s > Nr;
  {k,ts,Xs} = SimulationTaylor(X0,&pMu,&pSIGMA,Rho,0,5,Ns);
  Rs = Xs*alpha;
  CZsimul[.,s] = exp(-intTrapeze2(Rs,k));
  locate 5,5; print ftos(s,"Iteration No. %lf",2,0);
  s = s + 1;
endo;
  
CZsimul2 = meanc(CZsimul');
CZ = Solution(alpha,X0,MU,SIGMA,Rho,Lambda,ts);

graphset;
  begwind;
  window2(1,2,0,1.25);

  setwind(1);

    _pdate = ""; _ptitlht = 1.0; _pnum = 0; _paxes = 0;
    fonts("simplex simgrma complex");
    title("Modele lineaire gaussien avec deux facteurs\L"\
          "\202m\201]1[ = 0.3 \202m\201]2[ = 0.5 "\
          "\202s\201]1[ = 0.25 \202s\201]2[ = 0.15\L"\
          "\202l\201]1[ = 0.1 \202l\201]2[ = 0.2 "\
          "\202a\201]1[ = 0.3 \202a\201]2[ = -0.1 "); 
    draw;

  graphset;
    _pdate = ""; _pnum = 2; _pltype = 6|1; _plwidth = 0|0;
    _pnumht = 0.25; _ptitlht = 0.25; _paxht = 0.30;
    fonts("simplex simgrma complex");
    xlabel("\202t\201");
    xtics(0,5,1,10);

  setwind(2);
    title("1 simulation");
    _plegstr = "Prix theorique\000Prix simule";
    _plegctl = {2 6 2 1};
    ylabel("P[c](\202t\201)");
    xy(ts,CZ~CZsimul1);

  setwind(3);
    title("5000 simulations");
    _plegctl = 0;
    _pysci = 0;
    ylabel("erreur");
    xy(ts,CZ-CZsimul2);

    graphprt("-c=1 -cf=mc5.eps");
  endwind;
