new;
library tsm,optmum,spt,pgraph;
TSMset;


proc pMu(t,x,theta);
  retp(theta[1]*(theta[2]-x));
endp;

proc Grad_pMu(t,x,theta);
  local Nobs;
  Nobs = rows(x);
  retp( (theta[2]-x) ~ ((theta[1]~0).*ones(Nobs,1)) );
endp;

proc PSigma(t,x,theta);
  retp(theta[3]);
endp;

proc Grad_pSigma(t,x,theta);
  retp((0~0~1));
endp;

_eds_gradp = &Grad_pMu|&Grad_pSigma;


cls;

rndseed 456;

x0 = 0;
a = 0.8;
b = 0.1;
sigma = 0.06;
k = 1|0.5|0.1|0.05;
sv = a|b|sigma;

Nr = 250;  /* Nombre de replications */

__output = 0;
_print = 0;
_tsm_Mcov = 0;
_gmm_tol = 0.0001;

Nobs = 1000;
Est1000_a = zeros(Nr,4);
Est1000_b = zeros(Nr,4);
Est1000_sigma = zeros(Nr,4);

s1 = seqa(1,20,Nobs);
s2 = seqa(1,10,Nobs);
s3 = seqa(1,2,Nobs);
s4 = seqa(1,1,Nobs);

i = 1;
do until i > Nr;

  {t,x} = simulation_OU(x0,a,b,sigma,0,Nobs,20*Nobs,1);


  {theta,stderr,Mcov,Qmin} = gmm_eds(x[s1],&pMu,&pSigma,0,k[1],sv,0,0);

  Est1000_a[i,1] = theta[1];
  Est1000_b[i,1] = theta[2];
  Est1000_sigma[i,1] = theta[3];   

  {theta,stderr,Mcov,Qmin} = gmm_eds(x[s2],&pMu,&pSigma,0,k[2],sv,0,0);

  Est1000_a[i,2] = theta[1];
  Est1000_b[i,2] = theta[2];
  Est1000_sigma[i,2] = theta[3];   


  {theta,stderr,Mcov,Qmin} = gmm_eds(x[s3],&pMu,&pSigma,0,k[3],sv,0,0);

  Est1000_a[i,3] = theta[1];
  Est1000_b[i,3] = theta[2];
  Est1000_sigma[i,3] = theta[3];   


  {theta,stderr,Mcov,Qmin} = gmm_eds(x[s4],&pMu,&pSigma,0,k[4],sv,0,0);

  Est1000_a[i,4] = theta[1];
  Est1000_b[i,4] = theta[2];
  Est1000_sigma[i,4] = theta[3];   


  str = "1000 observations --- " $+ ftos(i,"Replication %lf",2,0);
  locate 5,5; print str;

  i = i + 1;
endo;



save simul20 = Est1000_a;
save simul21 = Est1000_b;
save simul22 = Est1000_sigma;


load Est1000_a = simul20;
load Est1000_b = simul21;
load Est1000_sigma = simul22;


X1 = {};
D1 = {};

_kernel[1:2] = 0.40|1.20;

i = 1;
do until i > 4;

  {xi,di,Fi,retcode} = Kernel(Est1000_a[.,i]);
  X1 = X1~xi;
  D1 = D1~di;

  i = i + 1;
endo;


X2 = {};
D2 = {};

_kernel[1:2] = 0.08|0.12;

i = 1;
do until i > 4;

  {xi,di,Fi,retcode} = Kernel(Est1000_b[.,i]);
  X2 = X2~xi;
  D2 = D2~di;

  i = i + 1;
endo;


X3 = {};
D3 = {};

_Kernel[1:2] = 0.035|0.065;

i = 1;
do until i > 4;

  {xi,di,Fi,retcode} = Kernel(abs(Est1000_sigma[.,i]));
  X3 = X3~xi;
  D3 = D3~di;

  i = i + 1;
endo;


graphset;
  begwind;
  window(2,2,0);

  setwind(1);

    _pdate = ""; _pnum = 0; _paxes = 0; _ptitlht = 0.30;
    fonts("simplex simgrma complex microb");
    _pmsgstr = "\203Densite empirique des estimateurs naifs\000"\
                   "(methode des moments generalises)\000"\
                   "pour le processus d'Ornstein-Uhlenbeck\000"\
                   "250 replications -- 1000 observations";
           
    _pmsgctl = 
               0.05~0.75~0.23~0~1~7~0 |
               0.05~0.6~0.23~0~1~7~0 |
               0.05~0.45~0.23~0~1~7~0 |
               0.05~0.2~0.23~0~1~7~0 ;            
    draw;

  graphset;
    _pnum = 2; _pdate = ""; _pnumht = 0.25; _ptitlht = 0.25; 
    _pltype = 6|1|3|4;
    fonts("simplex simgrma");
  
  setwind(2);

    xtics(0.4,1.2,0.2,2);
    ytics(0,7,1,0);
    title("\201parametre a");
    _pline = 1~6~0.8~0~0.8~1000~1~7~0;
    xy(X1,D1);

  setwind(3);
    xtics(0.08,0.12,0.01,0);
    ytics(0,150,50,5);
    title("\201parametre b");
    _pline = 1~6~0.10~0~0.10~1000~1~7~0;
    _plegstr = "k = 1\000k = 0.5\000k = 0.1\000k = 0.05";
    _plegctl = {2 7 7.5 2};
    xy(X2,D2);

  setwind(4);

    xtics(0.055,0.065,0.005,0);
    ytics(0,450,50,0);
    title("\201parametre \202s\201");
    _pline = 1~6~0.06~0~0.06~1000~1~7~0;
    _plegctl = {2 7 -1.5 2};
    xy(X3,D3);
   
    graphprt("-c=1 -cf=ou7.eps");

endwind;

